import { Node } from "@tiptap/core";
import { Attachment } from "../attachment";
import { TextDirections } from "../text-direction";
export interface ImageOptions {
    inline: boolean;
    allowBase64: boolean;
    HTMLAttributes: Record<string, unknown>;
}
/**
 * We have two attributes that store the source of an image:
 *
 * 1. `src`
 * 2. `dataurl`
 *
 * `src` is the image's inherent source. This can contain a URL, a base64-dataurl,
 * a blob...etc. We should never touch this attribute. This is also where we store
 * the data that we want to upload so after we download a pasted image, its base64
 * dataurl goes in this attribute.
 *
 * `dataurl` should never get added to the final HTML. This attribute is where we
 * restore an image's data after loading a note.
 *
 * The reason we have 2 instead of a single attribute is to avoid unnecessary processing.
 * Keeping everything in the `src` attribute requires us to always send the rendered image
 * along with everything else. This is pointless because we already have the image's rendered
 * data.
 */
export declare type ImageAttributes = Partial<ImageSizeOptions> & Partial<Attachment> & {
    src: string;
    dataurl?: string;
    alt?: string;
    title?: string;
    textDirection?: TextDirections;
    aspectRatio?: number;
};
export declare type ImageAlignmentOptions = {
    float?: boolean;
    align?: "center" | "left" | "right";
};
export declare type ImageSizeOptions = {
    width: number;
    height: number;
};
declare module "@tiptap/core" {
    interface Commands<ReturnType> {
        image: {
            /**
             * Add an image
             */
            insertImage: (options: Partial<ImageAttributes>) => ReturnType;
            updateImage: (query: {
                src?: string;
                hash?: string;
            }, options: Partial<ImageAttributes> & {
                preventUpdate?: boolean;
            }) => ReturnType;
            setImageAlignment: (options: ImageAlignmentOptions) => ReturnType;
            setImageSize: (options: ImageSizeOptions) => ReturnType;
        };
    }
}
export declare const inputRegex: RegExp;
export declare const ImageNode: Node<ImageOptions, any>;
