/*
This file is part of the Notesnook project (https://notesnook.com/)

Copyright (C) 2023 Streetwriters (Private) Limited

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import { Node, nodeInputRule, mergeAttributes, findChildren } from "@tiptap/core";
import { Plugin } from "@tiptap/pm/state";
import { hasSameAttributes } from "../../utils/prosemirror";
import { getDataAttribute } from "../attachment";
import { createSelectionBasedNodeView } from "../react";
import { ImageComponent } from "./component";
export const inputRegex = /(!\[(.+|:?)]\((\S+)(?:(?:\s+)["'](\S+)["'])?\))$/;
export const ImageNode = Node.create({
    name: "image",
    addOptions() {
        return {
            inline: true,
            allowBase64: true,
            HTMLAttributes: {}
        };
    },
    inline() {
        return this.options.inline;
    },
    group() {
        return this.options.inline ? "inline" : "block";
    },
    draggable: true,
    addAttributes() {
        return {
            src: {
                default: null
            },
            alt: {
                default: null
            },
            title: {
                default: null
            },
            width: { default: null },
            height: { default: null },
            // TODO: maybe these should be stored as styles?
            float: getDataAttribute("float", false),
            align: getDataAttribute("align"),
            hash: getDataAttribute("hash"),
            filename: getDataAttribute("filename"),
            type: getDataAttribute("mime"),
            size: getDataAttribute("size"),
            aspectRatio: {
                default: undefined,
                parseHTML: (element) => element.dataset.aspectRatio,
                renderHTML: (attributes) => {
                    if (!attributes.aspectRatio) {
                        return {};
                    }
                    return {
                        [`data-aspect-ratio`]: attributes.aspectRatio
                    };
                }
            },
            dataurl: Object.assign(Object.assign({}, getDataAttribute("dataurl")), { rendered: false })
        };
    },
    parseHTML() {
        return [
            {
                tag: this.options.allowBase64 ? "img" : 'img:not([src^="data:"])'
            }
        ];
    },
    renderHTML({ HTMLAttributes }) {
        return [
            "img",
            mergeAttributes(this.options.HTMLAttributes, HTMLAttributes)
        ];
    },
    addNodeView() {
        return createSelectionBasedNodeView(ImageComponent, {
            shouldUpdate: (prev, next) => !hasSameAttributes(prev.attrs, next.attrs)
        });
    },
    addProseMirrorPlugins() {
        return [
            new Plugin({
                props: {
                    transformCopied: (content) => {
                        content.content.descendants((node) => {
                            if (node.type.name === this.name &&
                                typeof node.attrs.dataurl === "string") {
                                const attrs = node.attrs;
                                attrs.src = attrs.dataurl;
                                delete attrs.dataurl;
                            }
                        });
                        return content;
                    }
                }
            })
        ];
    },
    addCommands() {
        return {
            insertImage: (options) => ({ commands }) => {
                return commands.insertContent({
                    type: this.name,
                    attrs: options
                });
            },
            setImageAlignment: (options) => ({ chain, tr }) => {
                const { from } = tr.selection;
                return chain()
                    .updateAttributes(this.name, Object.assign({}, options))
                    .setNodeSelection(from)
                    .run();
            },
            setImageSize: (options) => ({ chain, tr }) => {
                const { from } = tr.selection;
                return chain()
                    .updateAttributes(this.name, Object.assign({}, options))
                    .setNodeSelection(from)
                    .run();
            },
            updateImage: (query, options) => ({ state, tr }) => {
                const keyedQuery = query.hash
                    ? { key: "hash", value: query.hash }
                    : query.src
                        ? { key: "src", value: query.src }
                        : null;
                if (!keyedQuery)
                    return false;
                const images = findChildren(state.doc, (node) => node.type.name === this.name &&
                    node.attrs[keyedQuery.key] === keyedQuery.value);
                for (const image of images) {
                    tr.setNodeMarkup(image.pos, image.node.type, Object.assign(Object.assign({}, image.node.attrs), options));
                }
                tr.setMeta("preventUpdate", options.preventUpdate || false);
                tr.setMeta("addToHistory", false);
                return true;
            }
        };
    },
    addInputRules() {
        return [
            nodeInputRule({
                find: inputRegex,
                type: this.type,
                getAttributes: (match) => {
                    const [, , alt, src, title] = match;
                    return { src, alt, title };
                }
            })
        ];
    }
});
