/*
This file is part of the Notesnook project (https://notesnook.com/)

Copyright (C) 2023 Streetwriters (Private) Limited

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import { Node, mergeAttributes, findChildren, findParentNode } from "@tiptap/core";
import { findParentNodeOfTypeClosestToPos } from "../../utils/prosemirror";
import { onArrowUpPressed, onBackspacePressed } from "../list-item/commands";
import { OutlineList } from "../outline-list/outline-list";
export const OutlineListItem = Node.create({
    name: "outlineListItem",
    addOptions() {
        return {
            HTMLAttributes: {}
        };
    },
    addAttributes() {
        return {
            collapsed: {
                default: false,
                keepOnSplit: false,
                parseHTML: (element) => element.dataset.collapsed === "true",
                renderHTML: (attributes) => ({
                    "data-collapsed": attributes.collapsed === true
                })
            }
        };
    },
    content: "heading* paragraph block*",
    defining: true,
    parseHTML() {
        return [
            {
                tag: `li[data-type="${this.name}"]`
            }
        ];
    },
    renderHTML({ HTMLAttributes }) {
        return [
            "li",
            mergeAttributes(this.options.HTMLAttributes, HTMLAttributes, {
                "data-type": this.name
            }),
            0
        ];
    },
    addKeyboardShortcuts() {
        return {
            "Mod-Space": ({ editor }) => {
                const subList = findSublist(editor, this.type);
                if (!subList)
                    return false;
                const { isCollapsed, subListPos } = subList;
                return this.editor.commands.toggleOutlineCollapse(subListPos, !isCollapsed);
            },
            Enter: () => {
                // const subList = findSublist(editor, this.type);
                // if (!subList) return this.editor.commands.splitListItem(this.name);
                // const { isCollapsed, subListPos } = subList;
                // if (isCollapsed) {
                //   return this.editor.commands.toggleOutlineCollapse(subListPos, false);
                // }
                return this.editor.commands.splitListItem(this.name);
            },
            Tab: () => this.editor.commands.sinkListItem(this.name),
            "Shift-Tab": () => this.editor.commands.liftListItem(this.name),
            Backspace: ({ editor }) => onBackspacePressed(editor, this.name, this.type),
            ArrowUp: ({ editor }) => onArrowUpPressed(editor, this.name, this.type)
        };
    },
    addCommands() {
        return {
            toggleOutlineCollapse: (pos, state) => ({ tr }) => {
                tr.setNodeMarkup(pos, undefined, {
                    collapsed: state
                });
                return true;
            }
        };
    },
    addNodeView() {
        return ({ node, getPos, editor }) => {
            var _a;
            const isNested = ((_a = node.lastChild) === null || _a === void 0 ? void 0 : _a.type.name) === OutlineList.name;
            const li = document.createElement("li");
            if (node.attrs.collapsed)
                li.classList.add("collapsed");
            else
                li.classList.remove("collapsed");
            if (isNested)
                li.classList.add("nested");
            else
                li.classList.remove("nested");
            function onClick(e) {
                if (!(e.target instanceof HTMLParagraphElement))
                    return;
                if (!li.classList.contains("nested"))
                    return;
                const { x, y, right } = li.getBoundingClientRect();
                const clientX = e instanceof MouseEvent ? e.clientX : e.touches[0].clientX;
                const clientY = e instanceof MouseEvent ? e.clientY : e.touches[0].clientY;
                const hitArea = { width: 26, height: 24 };
                const selection = editor.state.selection;
                const parent = findParentNode((node) => !!node.attrs.textDirection)(selection);
                let xStart = clientX >= x - hitArea.width;
                let xEnd = clientX <= x;
                const yStart = clientY >= y;
                const yEnd = clientY <= y + hitArea.height;
                if (parent && parent.node.attrs.textDirection === "rtl") {
                    xEnd = clientX <= right + hitArea.width;
                    xStart = clientX >= right;
                }
                if (xStart && xEnd && yStart && yEnd) {
                    const pos = typeof getPos === "function" ? getPos() : 0;
                    if (!pos)
                        return;
                    e.preventDefault();
                    editor.commands.toggleOutlineCollapse(pos, !li.classList.contains("collapsed"));
                }
            }
            li.onmousedown = onClick;
            li.ontouchstart = onClick;
            return {
                dom: li,
                contentDOM: li,
                update: (updatedNode) => {
                    var _a;
                    if (updatedNode.type !== this.type) {
                        return false;
                    }
                    const isNested = ((_a = updatedNode.lastChild) === null || _a === void 0 ? void 0 : _a.type.name) === OutlineList.name;
                    if (updatedNode.attrs.collapsed)
                        li.classList.add("collapsed");
                    else
                        li.classList.remove("collapsed");
                    if (isNested)
                        li.classList.add("nested");
                    else
                        li.classList.remove("nested");
                    return true;
                }
            };
        };
    }
});
function findSublist(editor, type) {
    var _a, _b;
    const { selection } = editor.state;
    const { $from } = selection;
    const listItem = findParentNodeOfTypeClosestToPos($from, type);
    if (!listItem)
        return false;
    const [subList] = findChildren(listItem.node, (node) => node.type.name === OutlineList.name);
    if (!subList)
        return false;
    const isNested = ((_a = subList === null || subList === void 0 ? void 0 : subList.node) === null || _a === void 0 ? void 0 : _a.type.name) === OutlineList.name;
    const isCollapsed = (_b = subList === null || subList === void 0 ? void 0 : subList.node) === null || _b === void 0 ? void 0 : _b.attrs.collapsed;
    const subListPos = listItem.pos + subList.pos + 1;
    return { isCollapsed, isNested, subListPos };
}
