/*
This file is part of the Notesnook project (https://notesnook.com/)

Copyright (C) 2023 Streetwriters (Private) Limited

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * macOS uses its own Uniform Type Identifier format instead of mime-types.
 * This is an incomplete map of UTI->mime-type for image types taken from
 * https://gist.github.com/RhetTbull/7221ef3cfd9d746f34b2550d4419a8c2
 */
const UTITypes = {
    "com.microsoft.bmp": "image/bmp",
    "com.canon.crw-raw-image": "image/x-canon-crw",
    "com.adobe.raw-image": "image/x-adobe-dng",
    "com.dxo.raw-image": "image/x-dxo-dxo",
    "com.j2.efx-fax": "image/efax",
    "com.epson.raw-image": "image/x-epson-erf",
    "com.kodak.flashpix-image": "image/fpx",
    "com.compuserve.gif": "image/gif",
    "com.microsoft.ico": "image/vnd.microsoft.icon",
    "public.jpeg": "image/jpeg",
    "public.jpeg-2000": "image/jp2",
    "com.nikon.nrw-raw-image": "image/x-nikon-nrw",
    "com.apple.pict": "image/pict",
    "public.png": "image/png",
    "com.adobe.photoshop-image": "image/vnd.adobe.photoshop",
    "com.leica.pwl-raw-image": "image/x-leica-pwl",
    "com.apple.quicktime-image": "image/x-quicktime",
    "com.sgi.sgi-image": "image/sgi",
    "com.sony.sr2-raw-image": "image/x-sony-sr2",
    "public.svg-image": "image/svg+xml",
    "com.truevision.tga-image": "image/targa",
    "public.tiff": "image/tiff",
    "public.xbitmap-image": "image/x-xbitmap",
    "public.avci": "image/avci",
    "public.avcs": "image/avcs",
    "public.heic": "image/heic",
    "public.heif": "image/heif",
    "public.heics": "image/heic-sequence",
    "public.heifs": "image/heif-sequence"
};
export function downloadImage(url, options) {
    return __awaiter(this, void 0, void 0, function* () {
        if ((options === null || options === void 0 ? void 0 : options.corsHost) && !url.startsWith("blob:"))
            url = `${options.corsHost}/${url}`;
        const response = yield fetch(url);
        if (!response.ok)
            throw new Error(`invalid status code ${response.status}`);
        let contentType = response.headers.get("Content-Type");
        const contentLength = response.headers.get("Content-Length");
        if (contentType && UTITypes[contentType])
            contentType = UTITypes[contentType];
        if (!contentType ||
            !contentLength ||
            contentLength === "0" ||
            !contentType.startsWith("image/"))
            throw new Error("not an image");
        const size = parseInt(contentLength);
        let blob = yield response.blob();
        if (UTITypes[blob.type])
            blob = new Blob([blob], {
                type: contentType
            });
        return {
            blob,
            url: URL.createObjectURL(blob),
            type: contentType,
            size
        };
    });
}
export function toDataURL(blob) {
    return new Promise((resolve, reject) => {
        const reader = new FileReader();
        reader.onload = (_e) => resolve(reader.result);
        reader.onerror = (_e) => reject(reader.error);
        reader.onabort = (_e) => reject(new Error("Read aborted"));
        reader.readAsDataURL(blob);
    });
}
export function isDataUrl(url) {
    return (url === null || url === void 0 ? void 0 : url.startsWith("data:")) || false;
}
export function toBlobURL(dataurl) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!isDataUrl(dataurl))
            return dataurl;
        const response = yield fetch(dataurl);
        const blob = yield response.blob();
        return URL.createObjectURL(blob);
    });
}
