/*
This file is part of the Notesnook project (https://notesnook.com/)

Copyright (C) 2023 Streetwriters (Private) Limited

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import { findParentNode, findParentNodeClosestToPos, getChangedRanges } from "@tiptap/core";
import { BulletList } from "../extensions/bullet-list";
import { ListItem } from "../extensions/list-item";
import { OrderedList } from "../extensions/ordered-list";
import { OutlineList } from "../extensions/outline-list";
import { OutlineListItem } from "../extensions/outline-list-item";
import { TaskItemNode } from "../extensions/task-item";
import { TaskListNode } from "../extensions/task-list";
import { LIST_NODE_TYPES } from "./node-types";
export function hasSameAttributes(prev, next) {
    for (const key in prev) {
        const prevValue = prev[key];
        const nextValue = next[key];
        if (prevValue !== nextValue)
            return false;
    }
    return true;
}
export function findListItemType(editor) {
    const isTaskList = editor.isActive(TaskListNode.name);
    const isOutlineList = editor.isActive(OutlineList.name);
    const isList = editor.isActive(BulletList.name) || editor.isActive(OrderedList.name);
    return isList
        ? ListItem.name
        : isOutlineList
            ? OutlineListItem.name
            : isTaskList
                ? TaskItemNode.name
                : null;
}
export function isListActive(editor) {
    return LIST_NODE_TYPES.some((name) => editor.isActive(name));
}
export function findSelectedDOMNode(editor, types) {
    var _a;
    const { $anchor } = editor.state.selection;
    const selectedNode = editor.state.doc.nodeAt($anchor.pos);
    const pos = types.includes((selectedNode === null || selectedNode === void 0 ? void 0 : selectedNode.type.name) || "")
        ? $anchor.pos
        : (_a = findParentNode((node) => types.includes(node.type.name))(editor.state.selection)) === null || _a === void 0 ? void 0 : _a.pos;
    if (!pos)
        return null;
    return editor.view.nodeDOM(pos) || null;
}
export function findSelectedNode(editor, type) {
    var _a;
    const { $anchor } = editor.state.selection;
    const selectedNode = editor.state.doc.nodeAt($anchor.pos);
    const pos = (selectedNode === null || selectedNode === void 0 ? void 0 : selectedNode.type.name) === type
        ? $anchor.pos
        : (_a = findParentNode((node) => node.type.name === type)(editor.state.selection)) === null || _a === void 0 ? void 0 : _a.pos;
    if (pos === undefined)
        return null;
    return editor.state.doc.nodeAt(pos);
}
export function findMark(node, type) {
    const mark = node.marks.find((m) => m.type.name === type);
    return mark;
}
export function selectionToOffset(state) {
    const { $from, from } = state.selection;
    return {
        node: state.doc.nodeAt(from) || undefined,
        from,
        to: from + $from.node().nodeSize
    };
}
export const findChildren = (node, predicate, descend) => {
    if (!node) {
        throw new Error('Invalid "node" parameter');
    }
    else if (!predicate) {
        throw new Error('Invalid "predicate" parameter');
    }
    return walkNode(node, descend).filter((child) => predicate(child.node));
};
export function findChildrenByType(node, nodeType, descend = true) {
    return findChildren(node, (child) => child.type === nodeType, descend);
}
export const findParentNodeOfTypeClosestToPos = ($pos, nodeType) => {
    return findParentNodeClosestToPos($pos, (node) => equalNodeType(nodeType, node));
};
export function hasParentNode(predicate) {
    return function (selection) {
        return !!findParentNode(predicate)(selection);
    };
}
export function hasParentNodeOfType(nodeType) {
    return hasParentNode((node) => equalNodeType(nodeType, node));
}
export function findParentNodeOfType(nodeType) {
    return findParentNode((node) => equalNodeType(nodeType, node));
}
const walkNode = (node, descend = true) => {
    if (!node) {
        throw new Error('Invalid "node" parameter');
    }
    const result = [];
    node.descendants((child, pos) => {
        result.push({ node: child, pos });
        if (!descend) {
            return false;
        }
    });
    return result;
};
const equalNodeType = (nodeType, node) => {
    return ((Array.isArray(nodeType) && nodeType.indexOf(node.type) > -1) ||
        node.type === nodeType);
};
export function getChangedNodeRanges(tr) {
    // The container of the ranges to be returned from this function.
    const nodeRanges = [];
    const ranges = getChangedRanges(tr);
    for (const range of ranges) {
        try {
            const $from = tr.doc.resolve(range.newRange.from);
            const $to = tr.doc.resolve(range.newRange.to);
            // Find the node range for this provided range.
            const nodeRange = $from.blockRange($to);
            // Make sure a valid node is available.
            if (nodeRange) {
                nodeRanges.push(nodeRange);
            }
        }
        catch (_a) {
            // Changed ranged outside the document
        }
    }
    return nodeRanges;
}
/**
 * Get all the changed nodes from the provided transaction.
 *
 * The following example will give us all the text nodes in the provided
 * transaction.
 *
 * ```ts
 * import { getChangedNodes } from 'remirror/core';
 *
 * const changedTextNodes = getChangeNodes(tr, { descend: true, predicate: (node) => node.isText });
 * ```
 */
export function getChangedNodes(tr, options = {}) {
    const { descend = false, predicate } = options;
    const nodeRange = getChangedNodeRanges(tr);
    // The container for the nodes which have been added..
    const nodes = [];
    for (const range of nodeRange) {
        const { start, end } = range;
        // Find all the nodes between the provided node range.
        tr.doc.nodesBetween(start, end, (node, pos) => {
            var _a;
            // Check wether this is a node that should be added.
            const shouldAdd = (_a = predicate === null || predicate === void 0 ? void 0 : predicate(node, pos, range)) !== null && _a !== void 0 ? _a : true;
            if (shouldAdd) {
                nodes.push({ node, pos });
            }
            return descend;
        });
    }
    return nodes;
}
